<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class DLM_Buttons_Gutenberg {

	public function __construct() {
		add_action( 'init', array( $this, 'load' ) );
		add_action( 'wp_head', array( $this, 'print_inline_css' ) );
		add_filter( 'dlm_xhr_data', array( $this, 'xhr_links' ), 15, 1 );
	}

	public function print_inline_css() {
		$css = $this->generate_css();
		if( empty( $css ) ) return;
		printf( '<style id="dlm-buttons-css">%s</style>', wp_strip_all_tags( $css ));
	}

	public function load() {

		if ( ! function_exists( 'register_block_type' ) ) {
			// Gutenberg is not active.
			return;
		}

		$assets = array(
			'dependencies' =>
				array( 'react', 'react-dom', 'wp-api-fetch', 'wp-block-editor', 'wp-blocks', 'wp-components', 'wp-element', 'wp-i18n', 'wp-data' ),
			'version'      => '1.0.0',
		);

		// register Gutenberg JS
		wp_register_script(
			'dlm_buttons_gutenberg_block',
			DLM_BUTTONS_URL . 'assets/blocks/dist/blocks.js',
			$assets['dependencies'],
			$assets['version'],
			true
		);

		wp_register_style(
			'dlm_buttons_gutenberg_block-editor',
			DLM_BUTTONS_URL . 'assets/blocks/dist/blocks.css',
			array( 'wp-edit-blocks' )
		);

		// register the block in PHP
		register_block_type(
			'dlm-buttons/download-button',
			array(
				'editor_style'    => 'dlm_buttons_gutenberg_block-editor',
				'editor_script'   => 'dlm_buttons_gutenberg_block',
				'render_callback' => array( $this, 'render_download_button' ),
			)
		);

		wp_add_inline_script( 'dlm_buttons_gutenberg_block', 'const dlmButtonVars = {restURL:"' . esc_url_raw( get_rest_url( 'admin-ajax.php' ) ) . '",ajax_getVersions:"' . DLM_Ajax_Manager::get_ajax_url( 'get_versions' ) . '" };', 'before' );

		wp_set_script_translations( 'dlm_buttons_gutenberg_block', 'dlm-buttons', DLM_BUTTONS_URL . 'languages' );

	}

	public static function get_block_defaults() {
		return $attributes = array(
			'id' => 0,
			'title' => 'Download Button',
			);
	}

	public function render_download_button( $attributes, $content ) {

		$defaults = self::get_block_defaults();
		$attributes = wp_parse_args( $attributes, $defaults );

		if( ! isset( $attributes['id'] ) || 0 == $attributes['id']) {
			return '<p>'.esc_html__('Please select a download.', 'dlm-buttons').'</p>';
		}

		$icon_class = isset( $attributes['icon'] ) ? $attributes['icon'] : '';
		$icon_position = isset($attributes['iconPosition']) && 'right' == $attributes['iconPosition'] ? 'is-right-icon' : '';
		$remove_text = isset( $attributes['removeText'] ) && $attributes['removeText'] ? $attributes['removeText'] : false;
		$button_id = isset( $attributes['buttonId'] ) ? $attributes['buttonId'] : 0;

		try {
			/** @var DLM_Download $download */
			$download = download_monitor()->service( 'download_repository' )->retrieve_single( absint( $attributes['id'] ) );

			if ( isset( $attributes['version_id'] ) ) {

				try {
					/** @var DLM_Download_Version $version */
					$version = download_monitor()->service( 'version_repository' )->retrieve_single( absint( $attributes['version_id'] ) );
					$download->set_version( $version );
				} catch ( Exception $exception ) {
						// no version found, don't do anything.
				}
			}

		} catch ( Exception $exception ) {
			if ( current_user_can( 'manage_options' ) ) {

				return esc_html__( 'Please select a download.','download-monitor' );
			} else {

				return;
			}
		}

		$text = DLM_Buttons_Text::process_text( $attributes['title'], $download );

		if( isset( $attributes['template'] ) ) {
			// Render download button from template
			$template = plugin_dir_path( DLM_Buttons::get_plugin_file() ) . 'templates/button.php';
			$button_repo = new DLM_Buttons_Config_Repository();
			$args = array();
			try {
				$config                     = $button_repo->retrieve_single( $attributes['template'] );
				$args['dlm_buttons_config'] = $config;
			} catch ( Exception $e ) {
				// no action needed, the template will thrown an error because 'dlm_buttons_config' is missing
			}
			
			$args['dlm_download'] = $download;

			$template_handler = new DLM_Template_Handler();
			$template = $attributes['template'];
			ob_start();
			$template_handler->get_template_part('button', "dlm-buttons-$template", '', $args );
			return ob_get_clean();

		}
		if( isset( $attributes['icon'] )  ) {
			$svg_args = array(
				'svg'   => array(
					'class' => true,
					'aria-hidden' => true,
					'aria-labelledby' => true,
					'role' => true,
					'xmlns' => true,
					'width' => true,
					'height' => true,
					'viewbox' => true, // <= Must be lower case!
				),
				'g'     => array( 'fill' => true ),
				'title' => array( 'title' => true ),
				'path'  => array( 'd' => true, 'fill' => true,  ),
			);

		}
		ob_start();

		?>
		<div class="<?php echo "dlm-buttons-container-$button_id" ?>">
			<a href="<?php $download->the_download_link(); ?>" class="<?php echo "dlm-buttons-button dlm-buttons-button-$button_id" ?> <?php echo esc_attr( $icon_position ); ?>">
				<?php if( '' != $icon_class  ): ?>
					<span class="dlm-icon"><?php echo wp_kses( $attributes['icon'], $svg_args ); ?></span>					
				<?php endif; ?>
				<span><?php echo  $remove_text ? '' : wp_kses_post( $text ); ?></span>
			</a>
		</div><?php

		return ob_get_clean();
	}

	/**
	 * Parse our content for blocks.
	 *
	 * @param string $content Optional content to parse.
	 * @since 1.1
	 */
	public function dlm_buttons_get_parsed_content( $content = '' ) {

		if ( ! function_exists( 'has_blocks' ) || ! function_exists( 'parse_blocks' ) ) {
			return;
		}

		if ( ! $content && has_blocks( get_the_ID() ) ) {
			global $post;

			if ( ! is_object( $post ) ) {
				return;
			}

			$content = $post->post_content;
		}

		$content = parse_blocks( $content );

		return $content;
	}

	public function generate_css() {
		if ( ! function_exists( 'has_blocks' ) ) {
			return;
		}

		// $button_defaults = $this->get_button_style_defaults();

		$content = $this->dlm_buttons_get_parsed_content();

		if( !$content ) return;

		$css = '';
		$parsed_attr = array();
		$return = array();
		foreach ( $content as $name => $blockData ) {
			if( 'dlm-buttons/download-button' == $blockData['blockName'] ){
				$attributes = $blockData['attrs'];
				$parsed_attr = $this->parse_attributes( $attributes );
				$return[$parsed_attr['buttonId']] = $parsed_attr;
			}
		}

		foreach( $return as $button_id => $button_data ) {
			if( isset( $button_data['container'] )  ) {
				$css .= ".dlm-buttons-container-$button_id {";
				$css .= "display: flex;";
				$css .= $button_data['container'];
				$css .= "}";
			}
			if( isset( $button_data['normal'] ) ) {
				$css .= ".dlm-buttons-button-$button_id {";
				$css .= $button_data['normal'];
				$css .= "display: flex; width: fit-content; align-items: center; text-decoration: none;";
				$css .= "}";
			}
			if( isset( $button_data['hover'] ) ) {
				$css .= ".dlm-buttons-button-$button_id:hover {";
				$css .= $button_data['hover'];
				$css .= "}";
			}

			if( isset( $button_data['icon_style'] ) ) {
				$css .= ".dlm-buttons-button-$button_id .dlm-icon{";
				$css .= $button_data['span_icon_style'];
				$css .= "}";
				$css .= ".dlm-buttons-button-$button_id .dlm-icon svg {";
				$css .= $button_data['icon_style'];
				$css .= "}";
			}

			if( isset( $button_data['icon_hover'] ) ) {
				$css .= ".dlm-buttons-button-$button_id:hover .dlm-icon, .dlm-buttons-button-$button_id:hover svg {";
				$css .= $button_data['icon_hover'];
				$css .= "}";
			}

			$css .= ".dlm-buttons-button-$button_id.is-right-icon {";
			$css .= "flex-direction: row-reverse;";
			$css .= "}";
		}

		return $css;
	}

	public function parse_attributes( $attributes ) {
		$parsed_attr = array(
			'normal' => '',
			'hover' => '',
			'icon_style' => ''
		);
		$defaults_attr = $this->get_block_style_defaults();
		$attributes = wp_parse_args( $attributes, $defaults_attr );
		$skip_values = ['buttonId', 'templateConfig', 'id', 'icon'];
		foreach ( $attributes as $key => $value ) {

			if( in_array( $key, $skip_values ) ) {
				$parsed_attr[$key] = $value;
			}
			// Preg match capital letters
			$matches = preg_match_all('/[A-Z]/', $key, $matches);
			// If there is a match add a dash before each capital letter and lowercase the rest
			if( $matches ){
				$key = strtolower(preg_replace('/([a-z])([A-Z])/', '$1-$2', $key));
			}
			switch ( $key ) {
				case 'alignment' :
					$parsed_attr['container'] = "justify-content: {$value};";
				break;

				case 'font':
					$parsed_attr['normal'] .= "font-family: '{$value['fontFamily']}';";
					$parsed_attr['normal'] .= "font-size: {$value['fontSize']}{$value['after']};";
					$parsed_attr['normal'] .= 'font-weight: '.$value['fontWeight'].';';
					$parsed_attr['normal'] .= 'text-transform: '.$value['textTransform'].';';
				break;

				case 'margin':
				case 'padding':
				case 'border-radius':
					$parsed_attr['normal'] .= "$key: {$value['top']}{$value['after']} {$value['right']}{$value['after']} {$value['bottom']}{$value['after']} {$value['left']}{$value['after']};";
				break;
				case 'border-size':
					$parsed_attr['normal'] .= "border-top: {$value['top']}{$value['after']} solid;";
					$parsed_attr['normal'] .= "border-right: {$value['right']}{$value['after']} solid;";
					$parsed_attr['normal'] .= "border-bottom: {$value['bottom']}{$value['after']} solid;";
					$parsed_attr['normal'] .= "border-left: {$value['left']}{$value['after']} solid;";
				break;
				case 'colors':
					$parsed_attr['normal'] .= "background-color: {$value['backgroundColor']};";
					$parsed_attr['normal'] .= "color: {$value['textColor']};";
					$parsed_attr['normal'] .= "border-color: {$value['borderColor']};";

					$parsed_attr['hover'] .= "background-color: {$value['hoverBackgroundColor']};";
					$parsed_attr['hover'] .= "border-color: {$value['hoverBorderColor']};";
					$parsed_attr['hover'] .= "color: {$value['hoverTextColor']};";

					$parsed_attr['icon_style'] .= "color: {$value['iconColor']};";
					$parsed_attr['icon_style'] .= "fill: {$value['iconColor']};";
					// check if icon_hover is set if not set it
					if( !isset( $parsed_attr['icon_hover'] ) ) {
						$parsed_attr['icon_hover'] = '';
					}
					$parsed_attr['icon_hover'] .= "color: {$value['hoverIconColor']};";
					$parsed_attr['icon_hover'] .= "fill: {$value['hoverIconColor']};";
				break;

				case 'icon-size':
					$parsed_attr['icon_style'] .= "font-size: {$value['size']}{$value['after']};";
					$parsed_attr['icon_style'] .= "width: {$value['size']}{$value['after']};";
					$parsed_attr['icon_style'] .= "height: auto;";
				break;

				case 'icon-padding':
					$parsed_attr['span_icon_style'] = "width:auto;padding: {$value['top']}{$value['after']} {$value['right']}{$value['after']} {$value['bottom']}{$value['after']} {$value['left']}{$value['after']};";
				break;
			}


		}

		return $parsed_attr;
	}

	public function get_block_style_defaults() {
		return array (
			'alignment' => 'left',
			'font' => array (
				'fontFamily' => 'inherit',
				'fontWeight' => 'normal',
				'textTransform' => 'none',
				'fontSize' => 'inherit',
				'after' => ''
			),
			'padding' => array (
				'top' => '15',
				'right' => '15',
				'bottom' => '15',
				'left' => '15',
				'after' => 'px',
			),
			'margin' => array (
				'top' => '0',
				'right' => '0',
				'bottom' => '0',
				'left' => '0',
				'after' => 'px',
			),
			'borderSize' => array (
				'top' => '0',
				'right' => '0',
				'bottom' => '0',
				'left' => '0',
				'after' => 'px',
			),
			'borderRadius' => array (
				'top' => '0',
				'right' => '0',
				'bottom' => '0',
				'left' => '0',
				'after' => 'px',
			),
			'colors' => array (
				'backgroundColor' => '#0094C5',
				'textColor' => '#ffffff',
				'hoverBackgroundColor' => '#0094C5',
				'hoverBorderColor' => '#fff',
				'hoverTextColor' => '#fff',
				'iconColor' => '#ffffff',
				'hoverIconColor' => '#ffffff',
				'borderColor' => '#fff'
			),
			'iconPosition' => 'left',
			'iconSize' => array( 'size' => '24', 'after' => 'px' ),
			'iconPadding' => array( 'top' => '0', 'right' => '0', 'bottom' => '0', 'left' => '0', 'after' => 'px' ),
		);
	}

	/**
	 * Add XHR links
	 *
	 * @param array $xhr_data XHR data used by DLM to download files.
	 *
	 * @return array
	 * @since 4.0.6
	 *
	 */
	public function xhr_links( $xhr_data ) {
		$xhr_data['xhr_links']['class'][] = 'a[class^="dlm-buttons-button"]';
		$xhr_data['xhr_links']['class'][] = 'dlm-buttons-button';

		return $xhr_data;
	}
}

new DLM_Buttons_Gutenberg();
