<?php
/*
	Plugin Name: Download Monitor - Buttons
	Plugin URI: https://www.download-monitor.com/
	Description: The Buttons extension for Download Monitor allows you to create custom download buttons without any coding.
	Version: 4.2.0
	Author: WPChill
	Author URI: https://wpchill.com
	License: GPL v3
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

class DLM_Buttons {

	const VERSION = '4.2.0';

	public function __construct() {

		if ( class_exists( 'WP_DLM' ) ) {
			$this->add_files();
		}

		// assets.
		$assets = new DLM_Buttons_Assets();
		$assets->setup();

		// preview.
		$preview = new DLM_Buttons_Preview();
		$preview->setup();

		// template handler.
		$template_handler = new DLM_Buttons_Template_Handler();
		$template_handler->setup();

		// ajax.
		$ajax_save_template = new DLM_Buttons_Save_Template();
		$ajax_save_template->setup();

		$ajax_add_template = new DLM_Buttons_Add_Template();
		$ajax_add_template->setup();

		// Register Extension.
		add_filter( 'dlm_extensions', array( $this, 'register_extension' ) );

		// Admin only classes.
		if ( is_admin() ) {
			add_action( 'admin_notices', array( $this, 'admin_notices' ), 8 );
			// Add filter for compatibility, will be removed migration has been done.
			add_filter( 'dlm_disable_update_for_dlm-buttons', '__return_false', 15 );
			// Filter for uninstalling tables.
			add_filter( 'dlm_uninstall_db_tables', array( $this, 'uninstall_tables' ) );
			if ( ! $this->check_functionality() ) {
				return;
			}
			// The page
			$page = new DLM_Buttons_Admin_Page();

			$page->setup();

			// add settings
			//add_filter( 'download_monitor_settings', array( 'DLM_CA_Settings', 'add_settings' ) );

		}
	}

	/**
	 * Add required files
	 *
	 * @return void
	 * 
	 * @since 4.0.5
	 */
	public function add_files() {

		require_once __DIR__ . '/includes/Product/class-dlm-product-child.php';
		require_once __DIR__ . '/includes/class-dlm-extensions-notices.php';
		require_once __DIR__ . '/classes/class-dlm-buttons-gutenberg.php';
		require_once __DIR__ . '/classes/class-dlm-buttons-rest.php';
		require_once __DIR__ . '/classes/Elementor/class-dlm-buttons-elementor-widget-activation.php';
		require_once __DIR__ . '/includes/PROInstall/class-dlm-pro-install.php';
	}

	/**
	 * Get the plugin file
	 *
	 * @static
	 *
	 * @return String
	 */
	public static function get_plugin_file() {
		return __FILE__;
	}

	/**
	 * Register this extension
	 *
	 * @param array $extensions
	 *
	 * @return array $extensions
	 */
	public function register_extension( $extensions ) {

		$extensions[] = array(
			'file'    => 'dlm-buttons',
			'version' => DLM_Buttons::VERSION,
			'name'    => 'Buttons',
		);

		return $extensions;
	}

	/**
	 * Display admin notices
	 *
	 * @return void
	 */
	public function admin_notices() {

		if ( ! class_exists( 'WP_DLM' ) ) {
			?>
			<div class="error">
				<p><?php _e( 'Download Monitor - Buttons requires Download Monitor to work.', 'dlm-buttons' ); ?></p>
			</div>
			<?php
		}

	}

	/**
	 * Uninstall data
	 *
	 * @param array $tables Custom tables set by the AAM extension.
	 *
	 * @return array
	 * @since 4.0.7
	 */
	public function uninstall_tables( $tables ) {
		global $wpdb;
		$tables[] = $wpdb->prefix . 'dlm_buttons';

		return $tables;
	}

	/**
	 * Check extensions functionality
	 *
	 * @return bool
	 *
	 * @since 4.1.3
	 */
	public function check_functionality() {
		if ( class_exists( 'DLM_Admin_Helper' ) ) {
			$admin_helper = DLM_Admin_Helper::get_instance();
			if ( method_exists( $admin_helper, 'check_license_validity' ) ) {
				if ( ! $admin_helper->check_license_validity( 'dlm-buttons' ) ) {
					return false;
				}
			}
		}

		return true;
	}

}

require_once dirname( __FILE__ ) . '/vendor/autoload_52.php';

function __dlm_buttons() {

	define( 'DLM_BUTTONS_URL', plugin_dir_url( __FILE__ ) );
	define( 'DLM_BUTTONS_PATH', plugin_dir_path( __FILE__ ) );

	new DLM_Buttons();
}

add_action( 'plugins_loaded', '__dlm_buttons' );

if ( is_admin() && ( false === defined( 'DOING_AJAX' ) || false === DOING_AJAX ) ) {

	// set installer file constant
	define( 'DLM_BUTTONS_PLUGIN_FILE_INSTALLER', __FILE__ );

	// include installer functions
	require_once( 'includes/installer-functions.php' );

	// Activation hook
	register_activation_hook( DLM_BUTTONS_PLUGIN_FILE_INSTALLER, '_dlm_buttons_install' );

	// Multisite new blog hook
	add_action( 'wpmu_new_blog', '_dlm_buttons_mu_new_blog', 10, 6 );

	// Multisite blog delete
	add_filter( 'wpmu_drop_tables', '_dlm_buttons_mu_delete_blog' );

	register_deactivation_hook( DLM_BUTTONS_PLUGIN_FILE_INSTALLER, 'dlm_buttons_deactivate_plugin' );
	register_activation_hook( DLM_BUTTONS_PLUGIN_FILE_INSTALLER, 'dlm_buttons_activate_plugin' );

	function dlm_buttons_deactivate_plugin() {
		// Activate extension license if exists.
		if ( class_exists( 'DLM_PRO_Extensions_Handler' ) ) {
			$extensions = DLM_PRO_Extensions_Handler::get_instance();
			$extensions->handle_extension_action( 'deactivate', array(
				'slug' => 'dlm-buttons',
				'name' => 'Buttons'
			) );
		}
	}

	function dlm_buttons_activate_plugin() {
		// Activate extension license if exists.
		if ( class_exists( 'DLM_PRO_Extensions_Handler' ) ) {
			$extensions = DLM_PRO_Extensions_Handler::get_instance();
			$extensions->handle_extension_action( 'activate', array(
				'slug' => 'dlm-buttons',
				'name' => 'Buttons'
			) );
		}
	}
}